/**
 * Msg is an implementation for a message between nodes.
 *
 * @version 0.1
 * @author Bernhard Aichinger, Peter Praxmarer
 */

import daj.Message;

class Msg extends Message {

    public static final int TRY = 0;
    public static final int EXIT = 1;
    public static final int ACK = 2;
    public static final int OK = 3;
    public static final int ANY = 4;

    private int type;
    private int clock;
    private int proc;

    /**
     * Constructs a message according to the given parameters.
     *
     * @param type message type
     * @param clock clock value
     * @param proc process id
     */
    public Msg(int type, int clock, int proc) {
        this.type = type;
        this.clock = clock;
	this.proc = proc;
    }

    public String toString() {
	String s = "type=";
        
        switch (type) {
        case TRY: s += "TRY"; break;
        case EXIT: s += "EXIT"; break;
        case ACK: s += "ACK"; break;
        case OK: s += "OK"; break;
	case ANY: s += "ANY"; break;
        }
        
        s += ", clock=" + clock;
        s += ", proc=" + proc;
        return s;
    }

    public String getText() {
        String s = "type=";
        
        switch (type) {
        case TRY: s += "TRY"; break;
        case EXIT: s += "EXIT"; break;
        case ACK: s += "ACK"; break;
        case OK: s += "OK"; break;
        }
        
        s += " clock=" + clock;
        
        return s;
    }

    
    public int getType() {
        return type;
    }

    public void setType(int type) {
	this.type = type;
    }

    public int getClock() {
        return clock;
    }

    public void setClock(int clock) {
	this.clock = clock;
    }

    public int getProc() {
	return proc;
    }

    public boolean isTry() {
        return type == TRY;
    }

    
    public boolean isExit() {
        return type == EXIT;
    }


    public boolean isAck() {
        return type == ACK;
    }


    public boolean isOk() {
        return type == OK;
    }

    public boolean isBigger(Msg m) {
	return (clock > m.clock) || (clock == m.clock && proc > m.proc);
    }

    public boolean isSmaller(Msg m) {
	return (clock < m.clock) || (clock == m.clock && proc < m.proc);
    }
}
