import daj.*;


public class MutualExclusion extends Application {
    
    private static final int LOGICALTIMEME = 0;
    private static final int RICARTAGRAWALAME = 1;


    private static int
        algorithm = LOGICALTIMEME, // algorithm to use
        nodes = 3,                 // number of nodes
        width = 500;               // width and height of the window

    private static String title = "MutualExclusion - ";
    
    
    public static void main(String[] args) {

        if (args.length == 0) {
            System.out.println("Usage: RicartAgrawala -lt | -ra [-n nodes] [-w width]");
            System.out.println("  -lt \t use LogicalTimeME algorithm");
            System.out.println("  -ar \t use RicartAgrawalME algorithm");
            System.out.println("  -n nodes \t number of nodes to use");
            System.out.println("  -w width \t width and hight of window");
        }


        int i = 0;
        while (i < args.length) {
            if (args[i].equals("-lt"))
                algorithm = LOGICALTIMEME;

            else if (args[i].equals("-ra"))
                algorithm = RICARTAGRAWALAME;

            else if (args[i].equals("-n") && i + 1 < args.length) {
                nodes = new Integer(args[++i]).intValue();
                if (nodes < 2) nodes = 2;
            }

            else if (args[i].equals("-w") && i + 1 < args.length) {
                width = new Integer(args[++i]).intValue();
                if (width < 500) width = 500;
            }

            i++;
        }

        if (algorithm == LOGICALTIMEME)
            title += "LogcialTimeME";
        else
            title += "RicartAgrawalaME";

        new MutualExclusion().run();
    }

    
    public MutualExclusion() {
        super(title, width, width);
    }
    
    
    public void construct() {
        Node node[] = new Node[nodes];
        int radius = (width - 200) / 2;
        double delta = 2 * Math.PI / nodes;
        double rad;
        Program prog;
        
        // create the nodes, place them in a cycle
        rad = 0;
        for (int i = 0; i < nodes; i++) {
            if (algorithm == LOGICALTIMEME)
                prog = new LogicalTimeME(i, nodes);
            else
                prog = new RicartAgrawalaME(i, nodes);

            node[i] = node(prog,
                           (new Integer(i)).toString(),
                           (int)(width / 2 + radius * Math.cos(rad)),
                           (int)(width / 2 + radius * Math.sin(rad)));
            rad += delta;
        }
        
        // create channels between nodes
        for (int i = 0; i < nodes; i++) {
            for (int  j = 0; j < nodes; j++) {
                link(node[i], node[j]);
            }
        }
    }
    
    
    public String getText() {
        String description = "";

        if (algorithm == LOGICALTIMEME) {
            description += "LogicalTimeME";
        }
        else {
            description += "RicartAgrawalaME";
        }

        return description;
    }
    
}
